@@ -29,7 +29,6 @@ import org.apache.catalina.Globals;
 import org.apache.catalina.LifecycleException;
 import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Service;
-import org.apache.catalina.core.AprLifecycleListener;
 import org.apache.catalina.core.AprStatus;
 import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.coyote.AbstractProtocol;
@@ -37,38 +36,28 @@ import org.apache.coyote.Adapter;
 import org.apache.coyote.ProtocolHandler;
 import org.apache.coyote.UpgradeProtocol;
 import org.apache.coyote.http11.AbstractHttp11JsseProtocol;
-import org.apache.coyote.http11.AbstractHttp11Protocol;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.IntrospectionUtils;
 import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.CharsetUtil;
 import org.apache.tomcat.util.buf.EncodedSolidusHandling;
-import org.apache.tomcat.util.buf.StringUtils;
-import org.apache.tomcat.util.buf.UDecoder;
-import org.apache.tomcat.util.compat.JreCompat;
-import org.apache.tomcat.util.http.Method;
 import org.apache.tomcat.util.net.SSLHostConfig;
 import org.apache.tomcat.util.net.openssl.OpenSSLImplementation;
-import org.apache.tomcat.util.net.openssl.OpenSSLStatus;
 import org.apache.tomcat.util.res.StringManager;
 
 
 /**
  * Implementation of a Coyote connector.
+ *
+ * @author Craig R. McClanahan
+ * @author Remy Maucherat
  */
 public class Connector extends LifecycleMBeanBase {
 
     private static final Log log = LogFactory.getLog(Connector.class);
 
 
-    /**
-     * Alternate flag to enable recycling of facades.
-     */
-    public static final boolean RECYCLE_FACADES =
-            Boolean.parseBoolean(System.getProperty("org.apache.catalina.connector.RECYCLE_FACADES", "true"));
-
-
     public static final String INTERNAL_EXECUTOR_NAME = "Internal";
 
 
@@ -83,11 +72,10 @@ public class Connector extends LifecycleMBeanBase {
 
 
     public Connector(String protocol) {
-        boolean apr = AprStatus.getUseAprConnector() && AprStatus.isInstanceCreated() &&
-                AprLifecycleListener.isAprAvailable();
+        configuredProtocol = protocol;
         ProtocolHandler p = null;
         try {
-            p = ProtocolHandler.create(protocol, apr);
+            p = ProtocolHandler.create(protocol);
         } catch (Exception e) {
             log.error(sm.getString("coyoteConnector.protocolHandlerInstantiationFailed"), e);
         }
@@ -105,6 +93,7 @@ public class Connector extends LifecycleMBeanBase {
 
     public Connector(ProtocolHandler protocolHandler) {
         protocolHandlerClassName = protocolHandler.getClass().getName();
+        configuredProtocol = protocolHandlerClassName;
         this.protocolHandler = protocolHandler;
         // Default for Connector depends on this system property
         setThrowOnFailure(Boolean.getBoolean("org.apache.catalina.startup.EXIT_ON_INIT_FAILURE"));
@@ -119,6 +108,13 @@ public class Connector extends LifecycleMBeanBase {
     protected Service service = null;
 
 
+    /**
+     * If this is <code>true</code> the '\' character will be permitted as a path delimiter. If not specified, the
+     * default value of <code>false</code> will be used.
+     */
+    protected boolean allowBackslash = false;
+
+
     /**
      * Do we allow TRACE ?
      */
@@ -137,6 +133,16 @@ public class Connector extends LifecycleMBeanBase {
     protected boolean enableLookups = false;
 
 
+    /**
+     * If this is <code>true</code> then a call to <code>Response.getWriter()</code> if no character encoding has been
+     * specified will result in subsequent calls to <code>Response.getCharacterEncoding()</code> returning
+     * <code>ISO-8859-1</code> and the <code>Content-Type</code> response header will include a
+     * <code>charset=ISO-8859-1</code> component. (SRV.15.2.22.1) If not specified, the default specification compliant
+     * value of <code>true</code> will be used.
+     */
+    protected boolean enforceEncodingInGetWriter = true;
+
+
     /**
      * Is generation of X-Powered-By response header enabled/disabled?
      */
@@ -164,7 +170,7 @@ public class Connector extends LifecycleMBeanBase {
      * the object facades will be discarded when the request is recycled. If the security manager is enabled, this
      * setting is ignored and object facades are always discarded.
      */
-    protected boolean discardFacades = RECYCLE_FACADES;
+    protected boolean discardFacades = true;
 
 
     /**
@@ -198,22 +204,18 @@ public class Connector extends LifecycleMBeanBase {
 
     /**
      * The maximum number of parameters (GET plus POST) which will be automatically parsed by the container. 10000 by
-     * default. The default Tomcat server.xml configures a lower default of 1000. A value of less than 0 means no limit.
+     * default. A value of less than 0 means no limit.
      */
     protected int maxParameterCount = 10000;
 
-    private int maxPartCount = 50;
-
-    private int maxPartHeaderSize = 512;
-
     /**
-     * Maximum size of a POST which will be automatically parsed by the container. 2 MiB by default.
+     * Maximum size of a POST which will be automatically parsed by the container. 2MB by default.
      */
     protected int maxPostSize = 2 * 1024 * 1024;
 
 
     /**
-     * Maximum size of a POST which will be saved by the container during authentication. 4 KiB by default
+     * Maximum size of a POST which will be saved by the container during authentication. 4kB by default
      */
     protected int maxSavePostSize = 4 * 1024;
 
@@ -221,7 +223,7 @@ public class Connector extends LifecycleMBeanBase {
      * Comma-separated list of HTTP methods that will be parsed according to POST-style rules for
      * application/x-www-form-urlencoded request bodies.
      */
-    protected String parseBodyMethods = Method.POST;
+    protected String parseBodyMethods = "POST";
 
     /**
      * A Set of methods determined by {@link #parseBodyMethods}.
@@ -241,6 +243,12 @@ public class Connector extends LifecycleMBeanBase {
     protected final String protocolHandlerClassName;
 
 
+    /**
+     * Name of the protocol that was configured.
+     */
+    protected final String configuredProtocol;
+
+
     /**
      * Coyote protocol handler.
      */
@@ -260,17 +268,9 @@ public class Connector extends LifecycleMBeanBase {
 
 
     /**
-     * The behavior when an encoded reverse solidus (backslash - \) is submitted.
+     * The behavior when an encoded solidus (slash) is submitted.
      */
-    private EncodedSolidusHandling encodedReverseSolidusHandling = EncodedSolidusHandling.DECODE;
-
-
-    /**
-     * The behavior when an encoded solidus (slash - /) is submitted.
-     */
-    @SuppressWarnings("deprecation")
-    private EncodedSolidusHandling encodedSolidusHandling =
-            UDecoder.ALLOW_ENCODED_SLASH ? EncodedSolidusHandling.DECODE : EncodedSolidusHandling.REJECT;
+    private EncodedSolidusHandling encodedSolidusHandling = EncodedSolidusHandling.REJECT;
 
 
     /**
@@ -279,6 +279,9 @@ public class Connector extends LifecycleMBeanBase {
     protected boolean useBodyEncodingForURI = false;
 
 
+    private boolean rejectSuspiciousURIs;
+
+
     // ------------------------------------------------------------- Properties
 
     /**
@@ -312,35 +315,6 @@ public class Connector extends LifecycleMBeanBase {
     }
 
 
-    /**
-     * Return a property from the protocol handler.
-     *
-     * @param name the property name
-     *
-     * @return the property value
-     *
-     * @deprecated Use {@link #getProperty(String)}. This will be removed in Tomcat 10 onwards.
-     */
-    @Deprecated
-    public Object getAttribute(String name) {
-        return getProperty(name);
-    }
-
-
-    /**
-     * Set a property on the protocol handler.
-     *
-     * @param name  the property name
-     * @param value the property value
-     *
-     * @deprecated Use {@link #setProperty(String, String)}. This will be removed in Tomcat 10 onwards.
-     */
-    @Deprecated
-    public void setAttribute(String name, Object value) {
-        setProperty(name, String.valueOf(value));
-    }
-
-
     /**
      * @return the <code>Service</code> with which we are associated (if any).
      */
@@ -359,6 +333,24 @@ public class Connector extends LifecycleMBeanBase {
     }
 
 
+    /**
+     * @return <code>true</code> if backslash characters are allowed in URLs. Default value is <code>false</code>.
+     */
+    public boolean getAllowBackslash() {
+        return allowBackslash;
+    }
+
+
+    /**
+     * Set the allowBackslash flag.
+     *
+     * @param allowBackslash the new flag value
+     */
+    public void setAllowBackslash(boolean allowBackslash) {
+        this.allowBackslash = allowBackslash;
+    }
+
+
     /**
      * @return <code>true</code> if the TRACE method is allowed. Default value is <code>false</code>.
      */
@@ -374,7 +366,6 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void setAllowTrace(boolean allowTrace) {
         this.allowTrace = allowTrace;
-        setProperty("allowTrace", String.valueOf(allowTrace));
     }
 
 
@@ -430,7 +421,24 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void setEnableLookups(boolean enableLookups) {
         this.enableLookups = enableLookups;
-        setProperty("enableLookups", String.valueOf(enableLookups));
+    }
+
+
+    /**
+     * @return <code>true</code> if a default character encoding will be set when calling Response.getWriter()
+     */
+    public boolean getEnforceEncodingInGetWriter() {
+        return enforceEncodingInGetWriter;
+    }
+
+
+    /**
+     * Set the enforceEncodingInGetWriter flag.
+     *
+     * @param enforceEncodingInGetWriter the new flag value
+     */
+    public void setEnforceEncodingInGetWriter(boolean enforceEncodingInGetWriter) {
+        this.enforceEncodingInGetWriter = enforceEncodingInGetWriter;
     }
 
 
@@ -461,27 +469,6 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void setMaxParameterCount(int maxParameterCount) {
         this.maxParameterCount = maxParameterCount;
-        setProperty("maxParameterCount", String.valueOf(maxParameterCount));
-    }
-
-
-    public int getMaxPartCount() {
-        return maxPartCount;
-    }
-
-
-    public void setMaxPartCount(int maxPartCount) {
-        this.maxPartCount = maxPartCount;
-    }
-
-
-    public int getMaxPartHeaderSize() {
-        return maxPartHeaderSize;
-    }
-
-
-    public void setMaxPartHeaderSize(int maxPartHeaderSize) {
-        this.maxPartHeaderSize = maxPartHeaderSize;
     }
 
 
@@ -500,7 +487,6 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void setMaxPostSize(int maxPostSize) {
         this.maxPostSize = maxPostSize;
-        setProperty("maxPostSize", String.valueOf(maxPostSize));
     }
 
 
@@ -542,16 +528,15 @@ public class Connector extends LifecycleMBeanBase {
         HashSet<String> methodSet = new HashSet<>();
 
         if (null != methods) {
-            methodSet.addAll(Arrays.asList(StringUtils.splitCommaSeparated(methods)));
+            methodSet.addAll(Arrays.asList(methods.split("\\s*,\\s*")));
         }
 
-        if (methodSet.contains(Method.TRACE)) {
+        if (methodSet.contains("TRACE")) {
             throw new IllegalArgumentException(sm.getString("coyoteConnector.parseBodyMethodNoTrace"));
         }
 
         this.parseBodyMethods = methods;
         this.parseBodyMethodsSet = methodSet;
-        setProperty("parseBodyMethods", methods);
     }
 
 
@@ -633,17 +618,8 @@ public class Connector extends LifecycleMBeanBase {
     /**
      * @return the Coyote protocol handler in use.
      */
-    @SuppressWarnings("deprecation")
     public String getProtocol() {
-        boolean apr = AprStatus.getUseAprConnector();
-        if ((!apr && org.apache.coyote.http11.Http11NioProtocol.class.getName().equals(protocolHandlerClassName)) ||
-                (apr && org.apache.coyote.http11.Http11AprProtocol.class.getName().equals(protocolHandlerClassName))) {
-            return "HTTP/1.1";
-        } else if ((!apr && org.apache.coyote.ajp.AjpNioProtocol.class.getName().equals(protocolHandlerClassName)) ||
-                (apr && org.apache.coyote.ajp.AjpAprProtocol.class.getName().equals(protocolHandlerClassName))) {
-            return "AJP/1.3";
-        }
-        return protocolHandlerClassName;
+        return configuredProtocol;
     }
 
 
@@ -678,12 +654,11 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void setProxyName(String proxyName) {
 
-        if (proxyName != null && !proxyName.isEmpty()) {
+        if (proxyName != null && proxyName.length() > 0) {
             this.proxyName = proxyName;
         } else {
             this.proxyName = null;
         }
-        setProperty("proxyName", this.proxyName);
     }
 
 
@@ -702,7 +677,6 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void setProxyPort(int proxyPort) {
         this.proxyPort = proxyPort;
-        setProperty("proxyPort", String.valueOf(proxyPort));
     }
 
 
@@ -722,7 +696,6 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void setRedirectPort(int redirectPort) {
         this.redirectPort = redirectPort;
-        setProperty("redirectPort", String.valueOf(redirectPort));
     }
 
 
@@ -793,7 +766,8 @@ public class Connector extends LifecycleMBeanBase {
         try {
             Charset charset = B2CConverter.getCharset(URIEncoding);
             if (!CharsetUtil.isAsciiSuperset(charset)) {
-                log.error(sm.getString("coyoteConnector.notAsciiSuperset", URIEncoding));
+                log.error(sm.getString("coyoteConnector.notAsciiSuperset", URIEncoding, uriCharset.name()));
+                return;
             }
             uriCharset = charset;
         } catch (UnsupportedEncodingException e) {
@@ -817,7 +791,6 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void setUseBodyEncodingForURI(boolean useBodyEncodingForURI) {
         this.useBodyEncodingForURI = useBodyEncodingForURI;
-        setProperty("useBodyEncodingForURI", String.valueOf(useBodyEncodingForURI));
     }
 
     /**
@@ -839,7 +812,6 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void setXpoweredBy(boolean xpoweredBy) {
         this.xpoweredBy = xpoweredBy;
-        setProperty("xpoweredBy", String.valueOf(xpoweredBy));
     }
 
 
@@ -851,7 +823,6 @@ public class Connector extends LifecycleMBeanBase {
      */
     public void setUseIPVHosts(boolean useIPVHosts) {
         this.useIPVHosts = useIPVHosts;
-        setProperty("useIPVHosts", String.valueOf(useIPVHosts));
     }
 
 
@@ -894,21 +865,6 @@ public class Connector extends LifecycleMBeanBase {
     }
 
 
-    public String getEncodedReverseSolidusHandling() {
-        return encodedReverseSolidusHandling.getValue();
-    }
-
-
-    public void setEncodedReverseSolidusHandling(String encodedReverseSolidusHandling) {
-        this.encodedReverseSolidusHandling = EncodedSolidusHandling.fromString(encodedReverseSolidusHandling);
-    }
-
-
-    public EncodedSolidusHandling getEncodedReverseSolidusHandlingInternal() {
-        return encodedReverseSolidusHandling;
-    }
-
-
     public String getEncodedSolidusHandling() {
         return encodedSolidusHandling.getValue();
     }
@@ -924,6 +880,16 @@ public class Connector extends LifecycleMBeanBase {
     }
 
 
+    public boolean getRejectSuspiciousURIs() {
+        return rejectSuspiciousURIs;
+    }
+
+
+    public void setRejectSuspiciousURIs(boolean rejectSuspiciousURIs) {
+        this.rejectSuspiciousURIs = rejectSuspiciousURIs;
+    }
+
+
     // --------------------------------------------------------- Public Methods
 
     /**
@@ -979,7 +945,7 @@ public class Connector extends LifecycleMBeanBase {
             } else if (addressObj != null) {
                 address = addressObj.toString();
             }
-            if (!address.isEmpty()) {
+            if (address.length() > 0) {
                 sb.append(",address=");
                 sb.append(ObjectName.quote(address));
             }
@@ -1016,7 +982,6 @@ public class Connector extends LifecycleMBeanBase {
     }
 
 
-    @SuppressWarnings("deprecation")
     @Override
     protected void initInternal() throws LifecycleException {
 
@@ -1029,38 +994,21 @@ public class Connector extends LifecycleMBeanBase {
         // Initialize adapter
         adapter = new CoyoteAdapter(this);
         protocolHandler.setAdapter(adapter);
+        if (service != null) {
+            protocolHandler.setUtilityExecutor(service.getServer().getUtilityExecutor());
+        }
 
         // Make sure parseBodyMethodsSet has a default
         if (null == parseBodyMethodsSet) {
             setParseBodyMethods(getParseBodyMethods());
         }
 
-        if (protocolHandler.isAprRequired() && !AprStatus.isInstanceCreated()) {
-            throw new LifecycleException(
-                    sm.getString("coyoteConnector.protocolHandlerNoAprListener", getProtocolHandlerClassName()));
-        }
-        if (protocolHandler.isAprRequired() && !AprStatus.isAprAvailable()) {
-            throw new LifecycleException(
-                    sm.getString("coyoteConnector.protocolHandlerNoAprLibrary", getProtocolHandlerClassName()));
-        }
-        if (!protocolHandler.isAprRequired()) {
-            if (JreCompat.isJre22Available() && OpenSSLStatus.getUseOpenSSL() && OpenSSLStatus.isAvailable() &&
-                    protocolHandler instanceof AbstractHttp11Protocol) {
-                // Use FFM and OpenSSL if available
-                AbstractHttp11JsseProtocol<?> jsseProtocolHandler = (AbstractHttp11JsseProtocol<?>) protocolHandler;
-                if (jsseProtocolHandler.isSSLEnabled() && jsseProtocolHandler.getSslImplementationName() == null) {
-                    // OpenSSL is compatible with the JSSE configuration, so use it if it is available
-                    jsseProtocolHandler.setSslImplementationName(
-                            "org.apache.tomcat.util.net.openssl.panama.OpenSSLImplementation");
-                }
-            } else if (AprStatus.isAprAvailable() && AprStatus.getUseOpenSSL() &&
-                    protocolHandler instanceof AbstractHttp11Protocol) {
-                // Use tomcat-native and OpenSSL otherwise, if available
-                AbstractHttp11JsseProtocol<?> jsseProtocolHandler = (AbstractHttp11JsseProtocol<?>) protocolHandler;
-                if (jsseProtocolHandler.isSSLEnabled() && jsseProtocolHandler.getSslImplementationName() == null) {
-                    // OpenSSL is compatible with the JSSE configuration, so use it if APR is available
-                    jsseProtocolHandler.setSslImplementationName(OpenSSLImplementation.class.getName());
-                }
+        if (AprStatus.isAprAvailable() && AprStatus.getUseOpenSSL() &&
+                protocolHandler instanceof AbstractHttp11JsseProtocol) {
+            AbstractHttp11JsseProtocol<?> jsseProtocolHandler = (AbstractHttp11JsseProtocol<?>) protocolHandler;
+            if (jsseProtocolHandler.isSSLEnabled() && jsseProtocolHandler.getSslImplementationName() == null) {
+                // OpenSSL is compatible with the JSSE configuration, so use it if APR is available
+                jsseProtocolHandler.setSslImplementationName(OpenSSLImplementation.class.getName());
             }
         }
 
@@ -1089,15 +1037,9 @@ public class Connector extends LifecycleMBeanBase {
 
         setState(LifecycleState.STARTING);
 
-        // Configure the utility executor before starting the protocol handler
-        if (protocolHandler != null && service != null) {
-            protocolHandler.setUtilityExecutor(service.getServer().getUtilityExecutor());
-        }
-
         try {
             protocolHandler.start();
         } catch (Exception e) {
-            // Includes NPE - protocolHandler will be null for invalid protocol if throwOnFailure is false
             throw new LifecycleException(sm.getString("coyoteConnector.protocolHandlerStartFailed"), e);
         }
     }
@@ -1120,11 +1062,6 @@ public class Connector extends LifecycleMBeanBase {
         } catch (Exception e) {
             throw new LifecycleException(sm.getString("coyoteConnector.protocolHandlerStopFailed"), e);
         }
-
-        // Remove the utility executor once the protocol handler has been stopped
-        if (protocolHandler != null) {
-            protocolHandler.setUtilityExecutor(null);
-        }
     }
 
 
@@ -1146,28 +1083,27 @@ public class Connector extends LifecycleMBeanBase {
     }
 
 
+    /**
+     * Provide a useful toString() implementation as it may be used when logging Lifecycle errors to identify the
+     * component.
+     */
     @Override
     public String toString() {
         // Not worth caching this right now
         StringBuilder sb = new StringBuilder("Connector[");
-        String name = (String) getProperty("name");
-        if (name == null) {
-            sb.append(getProtocol());
-            sb.append('-');
-            String id = (protocolHandler != null) ? protocolHandler.getId() : null;
-            if (id != null) {
-                sb.append(id);
-            } else {
-                int port = getPortWithOffset();
-                if (port > 0) {
-                    sb.append(port);
-                } else {
-                    sb.append("auto-");
-                    sb.append(getProperty("nameIndex"));
-                }
-            }
+        sb.append(getProtocol());
+        sb.append('-');
+        String id = (protocolHandler != null) ? protocolHandler.getId() : null;
+        if (id != null) {
+            sb.append(id);
         } else {
-            sb.append(name);
+            int port = getPortWithOffset();
+            if (port > 0) {
+                sb.append(port);
+            } else {
+                sb.append("auto-");
+                sb.append(getProperty("nameIndex"));
+            }
         }
         sb.append(']');
         return sb.toString();
